@@ -1,4 +1,4 @@
-c------------------------------------------ --------------------------*
+c--------------------------------------------------------------------*
 c SPARSITY of 1-D PDE problems
 c--------------------------------------------------------------------*
 
@@ -94,6 +94,7 @@ c Determines the sparsity structure of the Jacobian in 2-D reaction- *
 c transport problems.                                                *
 c                                                                    *
 C IAN  and JAN: see comments in sparse1D                             *
+C A(I,J) in vector: at position (I-1)*dim(2) + J                     *
 C                                                                    *
 c--------------------------------------------------------------------*
 
@@ -134,7 +135,7 @@ c interactions with current, upstream and downstream boxes
               jan(ij) = M+1
               ij      = ij +1
             ELSE IF (cyclic(2) == 1) THEN
-              jan(ij) = isp + (j-1)*dimens(2) +1
+              jan(ij) = isp + (j-1)*dimens(2) +1    
               ij      = ij +1
             ENDIF
 
@@ -143,7 +144,7 @@ c interactions with current, upstream and downstream boxes
               ij      = ij +1
             ELSE IF (cyclic(1) == 1) THEN
               jan(ij) = isp + K
-              ij      = ij +1
+              ij      = ij +1  
             ENDIF
 
             IF (j >1) THEN
@@ -158,7 +159,7 @@ c interactions with current, upstream and downstream boxes
               jan(ij) = M-1
               ij      = ij +1
             ELSE IF (cyclic(2) == 1) THEN
-              jan(ij) = isp + j*dimens(2)
+              jan(ij) = isp + j*dimens(1)
               ij      = ij +1
             ENDIF
 
@@ -177,6 +178,33 @@ c interactions with other species in the same box
 c
       END SUBROUTINE sparse2d
 
+      SUBROUTINE interact (ij, NNZ, ian, jan, M, ival)
+      INTEGER IJ, NNZ, ival, ian(*), jan(*), i, isave
+	character *80 msg
+
+
+      isave = 1
+c check if not yet present for current state
+	DO I = ian(M), ij-1
+	  IF (jan(I) .EQ. ival) THEN
+	     isave = 0
+	     exit
+	  ENDIF
+	ENDDO
+
+c save 
+	IF (isave .EQ. 1) THEN
+        IF (ij .GT. nnz) THEN
+          WRITE (msg,*)
+     &("cannot generate sparse jacobian - not enough room for nonzeros")
+          CALL rexit(msg)
+        ENDIF
+        jan(ij) = ival
+        ij = ij +1      
+      ENDIF
+	 
+      END SUBROUTINE interact
+       
 c--------------------------------------------------------------------*
 c SPARSITY of 3-D PDE problems
 c--------------------------------------------------------------------*
@@ -189,6 +217,7 @@ c problems.                                                          *
 c                                                                    *
 C IAN  and JAN: see comments in sparse1D                             *
 C                                                                    *
+C A(I,J,K) in vector: (I-1)*dim(2)*dim(3) + (J-1)*dim(3) + K         *  
 c--------------------------------------------------------------------*
 
 c total number of state variables, number of different species
@@ -198,7 +227,7 @@ c dimensions of the problem and whether cyclic boundaries or not
 c maximal number of indices, sparsity arrays
        INTEGER nnz, ian(*), jan(*)
 c
-       INTEGER N, I, J, ij, K, L, M , isp
+       INTEGER N, I, J, ij, K, L, M, isp, im
  	     character *80 msg
 
 c check input
@@ -209,7 +238,7 @@ c check input
        ENDIF
 
 c number of boxes
-       N = dimens(1)*dimens(2)
+       N = dimens(1)*dimens(2)*dimens(3)
 
        ij     = 1
        ian(1) = 1
@@ -221,53 +250,70 @@ c number of boxes
              DO ll = 1, dimens(3)
                M = isp +(j-1)*dimens(2)*dimens(3)+(k-1)*dimens(3)+ll
 
-c interactions with current, upstream and downstream boxes
-              jan(ij) = M
-              ij      = ij +1
+c interactions with current, upstream and downstream boxes (ival = M, M+1, M-1)
+               CALL interact(ij, nnz, ian, jan, M, M) 
 
-              IF (ll<dimens(3)) THEN
-                jan(ij) = M+1
-                ij      = ij +1
-              ENDIF
-              IF (ll >1) THEN
-                jan(ij) = M-1
-                ij      = ij +1
-              ENDIF
+               IF (ll<dimens(3)) THEN
+                 CALL interact(ij, nnz, ian, jan, M, M+1) 
+
+               ELSE IF (cyclic(3) == 1 .AND. dimens(3) > 2) THEN
+	           im = isp + (j-1)*dimens(2)*dimens(3)+(k-1)*dimens(3)+1   
+                 CALL interact(ij, nnz, ian, jan, M, im) 
+               ENDIF
+              
+			 IF (ll >1) THEN
+                 CALL interact(ij, nnz, ian, jan, M, M-1) 
+               ELSE IF (cyclic(3) == 1 .AND. dimens(3) > 2) THEN
+                 im = isp + (j-1)*dimens(2)*dimens(3)+k*dimens(3) 
+                 CALL interact(ij, nnz, ian, jan, M, im) 
+               ENDIF
  
-              IF (k<dimens(2)) THEN
-                jan(ij) = M+dimens(3)
-                ij      = ij +1
-              ENDIF
+               IF (k<dimens(2)) THEN
+                 CALL interact(ij, nnz, ian, jan, M, M+dimens(3)) 
+               ELSE IF (cyclic(2) == 1 .AND. dimens(2) > 2) THEN
+                 im = isp + (j-1)*dimens(2)*dimens(3)+ll 
+                 CALL interact(ij, nnz, ian, jan, M, im) 
+               ENDIF
             
-              IF (k >1) THEN
-                jan(ij) = M-dimens(3)
-                ij      = ij +1
-              ENDIF
+               IF (k >1) THEN
+                 CALL interact(ij, nnz, ian, jan, M, M -dimens(3))
+               ELSE IF (cyclic(2) == 1 .AND. dimens(2) > 2) THEN
+                 im = isp +j*dimens(2)*dimens(3) -dimens(3)+ll
+                 CALL interact(ij, nnz, ian, jan, M, im) 
+               ENDIF
 
-              IF (j<dimens(1)) THEN
-                jan(ij) = M+dimens(2)*dimens(3)
-                ij      = ij +1
-              ENDIF
+               IF (j<dimens(1)) THEN
+                 im = M+dimens(2)*dimens(3)
+                 CALL interact(ij, nnz, ian, jan, M, im) 
+               ELSE IF (cyclic(1) == 1 .AND. dimens(1) > 2) THEN
+                 im = isp +(k-1)*dimens(3)+ll
+                 CALL interact(ij, nnz, ian, jan, M, im) 
+               ENDIF
 
-              IF (j >1) THEN
-                jan(ij) = M-dimens(2)*dimens(3)
-                ij      = ij +1
-              ENDIF
+               IF (j >1) THEN
+                 im = M-dimens(2)*dimens(3)
+                 CALL interact(ij, nnz, ian, jan, M, im) 
+               ELSE IF (cyclic(1) == 1 .AND. dimens(1) > 2) THEN
+	           im = isp +(dimens(1)-1)*dimens(2)*dimens(3)+                    &
+     &		            (k-1)*dimens(3)+ll
+                 CALL interact(ij, nnz, ian, jan, M, im) 
+               ENDIF
 
 
 c interactions with other species in the same box
-              DO L = 1, Nspec
-                IF (L == i) cycle
-                jan(ij)=(L-1)*N+(j-1)*dimens(2)*dimens(3)+                       &
-     &                     (k-1)*dimens(3)+ll
-                ij = ij +1
-              ENDDO
+               DO L = 1, Nspec
+                 IF (L == i) cycle
+                 im=(L-1)*N+(j-1)*dimens(2)*dimens(3)+                           &
+     &                      (k-1)*dimens(3)+ll
+                 CALL interact(ij, nnz, ian, jan, M, im) 
+               ENDDO
 
-              ian(M+1) = ij
-            ENDDO
-	    ENDDO
-        ENDDO
+               ian(M+1) = ij
+             ENDDO
+	     ENDDO
+         ENDDO
       ENDDO
       nnz = ij -1
+
 c
-      END SUBROUTINE sparse3d
\ No newline at end of file
+      END SUBROUTINE sparse3d
